@@ -1,5 +1,5 @@
 #
-# Copyright 2018-2021 Elyra Authors
+# Copyright 2018-2022 Elyra Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,9 +41,10 @@ class TypeHintRemover(ast.NodeTransformer):
 
 
 class AirflowComponentParser(ComponentParser):
-    _component_platform: RuntimeProcessorType = RuntimeProcessorType.APACHE_AIRFLOW
     _file_types: List[str] = [".py"]
 
+    component_platform: RuntimeProcessorType = RuntimeProcessorType.APACHE_AIRFLOW
+
     def parse(self, registry_entry: SimpleNamespace) -> Optional[List[Component]]:
         components: List[Component] = list()
 
@@ -171,6 +172,7 @@ class AirflowComponentParser(ComponentParser):
                                                   default_control_type=default_control_type,
                                                   control_id=control_id,
                                                   one_of_control_types=one_of_control_types,
+                                                  allow_no_options=True,
                                                   required=required)
             properties.append(component_params)
 
