@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2021 Elyra Authors
+ * Copyright 2018-2022 Elyra Authors
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -76,8 +76,7 @@ export interface IMetadataDisplayProps {
   schemaspace: string;
   sortMetadata: boolean;
   className: string;
-  // Optional string to append to a schema display name
-  schemaType?: string;
+  titleContext?: string;
   labelName?: (args: any) => string;
   omitTags?: boolean;
 }
@@ -115,7 +114,7 @@ export class MetadataDisplay<
     return showDialog({
       title: `Delete ${
         this.props.labelName ? this.props.labelName(metadata) : ''
-      } '${metadata.display_name}'?`,
+      } ${this.props.titleContext || ''} '${metadata.display_name}'?`,
       buttons: [Dialog.cancelButton(), Dialog.okButton()]
     }).then((result: any) => {
       // Do nothing if the cancel button is pressed
@@ -326,8 +325,8 @@ export interface IMetadataWidgetProps {
   display_name: string;
   schemaspace: string;
   icon: LabIcon;
-  // Optional string to append after schema display name
-  schemaType?: string;
+  titleContext?: string;
+  appendToTitle?: boolean;
 }
 
 /**
@@ -337,7 +336,7 @@ export class MetadataWidget extends ReactWidget {
   renderSignal: Signal<this, any>;
   props: IMetadataWidgetProps;
   schemas?: IDictionary<any>[];
-  schemaType?: string;
+  titleContext?: string;
 
   constructor(props: IMetadataWidgetProps) {
     super();
@@ -345,8 +344,7 @@ export class MetadataWidget extends ReactWidget {
 
     this.props = props;
     this.renderSignal = new Signal<this, any>(this);
-    this.schemaType = props.schemaType;
-
+    this.titleContext = props.titleContext;
     this.fetchMetadata = this.fetchMetadata.bind(this);
     this.updateMetadata = this.updateMetadata.bind(this);
     this.openMetadataEditor = this.openMetadataEditor.bind(this);
@@ -428,6 +426,7 @@ export class MetadataWidget extends ReactWidget {
         sortMetadata={true}
         className={`${METADATA_CLASS}-${this.props.schemaspace}`}
         omitTags={this.omitTags()}
+        titleContext={this.props.titleContext}
       />
     );
   }
@@ -450,7 +449,8 @@ export class MetadataWidget extends ReactWidget {
             <AddMetadataButton
               schemas={this.schemas}
               addMetadata={this.addMetadata}
-              schemaType={this.schemaType}
+              titleContext={this.titleContext}
+              appendToTitle={this.props.appendToTitle}
             />
           </header>
           <UseSignal signal={this.renderSignal} initialArgs={[]}>
